package com.faraphel.tasks_valider.connectivity.bwd

import android.Manifest
import android.app.Activity
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.net.wifi.p2p.*
import android.util.Log
import androidx.compose.runtime.mutableStateOf
import com.faraphel.tasks_valider.connectivity.bwd.error.*


/**
 * A helper to wrap the WiFi-Direct manager, the channel and the events.
 *
 * This avoids certain annoying features such as always specifying the channel as the first argument or
 * handling all the events with the base event system.
 *
 * @param manager The WiFi-Direct manager
 * @param channel The WiFi-Direct channel
 */
class BwdManager(
    private var manager: WifiP2pManager,
    private var channel: WifiP2pManager.Channel,
) : BroadcastReceiver() {
    companion object {
        var PERMISSION_ACCESS_FINE_LOCATION = 1001  ///< permission code for the fine location
        var ALL_INTENT_FILTER = IntentFilter().apply {
            addAction(WifiP2pManager.WIFI_P2P_CONNECTION_CHANGED_ACTION)
            addAction(WifiP2pManager.WIFI_P2P_PEERS_CHANGED_ACTION)
        }

        fun isSupported(context: Context): Boolean {
            return context.packageManager.hasSystemFeature(PackageManager.FEATURE_WIFI_DIRECT)
        }

        /**
         * Create a new BwfManager from an activity.
         * @param activity The activity to create the manager from
         */
        fun fromActivity(activity: Activity): BwdManager {
            // check if the system support WiFi-Direct
            if (this.isSupported(activity)) {
                Log.e("wifi-p2p", "this device does not support the WiFi-Direct feature")
                throw BwdNotSupportedException()
            }

            // TODO(Faraphel): more check on permissions
            if (
                activity.checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION) != PackageManager.PERMISSION_GRANTED ||
                activity.checkSelfPermission(Manifest.permission.NEARBY_WIFI_DEVICES) != PackageManager.PERMISSION_GRANTED
            ) {
                // TODO(Faraphel): should be used with shouldShowRequestPermissionRationale, with a check
                activity.requestPermissions(
                    arrayOf(Manifest.permission.ACCESS_FINE_LOCATION),
                    PERMISSION_ACCESS_FINE_LOCATION
                )
            }

            // get the WiFi-Direct manager
            val manager = activity.getSystemService(Context.WIFI_P2P_SERVICE) as WifiP2pManager?
                ?: throw BwdPermissionException()

            // get the WiFi-Direct channel
            val channel = manager.initialize(activity, activity.mainLooper, null)
            return BwdManager(manager, channel)

            // NOTE(Faraphel): the broadcast receiver should be registered in the activity onResume
        }
    }

    // Wrappers

    /**
     * Connect to another device, allowing for a communication using Sockets
     * @see WifiP2pManager.connect
     * @throws SecurityException if the permission has not been given
     */
    @Throws(SecurityException::class)
    fun connect(config: WifiP2pConfig, callback: () -> Unit = {}) =
        this.manager.connect(this.channel, config, object : WifiP2pManager.ActionListener {
            override fun onSuccess() { callback() }
            override fun onFailure(reason: Int) = throw BwdConnectException(reason)
        })

    /**
     * Request the list of peers after a discovery.
     * @see WifiP2pManager.requestPeers
     * @throws SecurityException if the permission has not been given
     */
    @Throws(SecurityException::class)
    fun requestPeers(callback: (WifiP2pDeviceList) -> Unit = {}) =
        this.manager.requestPeers(this.channel, callback)

    /**
     * Start discovering peers.
     * Once founds, the WIFI_P2P_PEERS_CHANGED_ACTION event will be triggered.
     * @see WifiP2pManager.discoverPeers
     * @throws SecurityException if the permission has not been given
     */
    @Throws(SecurityException::class)
    fun discoverPeers(callback: () -> Unit = {}) =
        this.manager.discoverPeers(this.channel, object : WifiP2pManager.ActionListener {
            override fun onSuccess() { callback() }
            override fun onFailure(reason: Int) = throw BwdDiscoverException(reason)
        })

    /**
     * Obtain information about a connection with another device.
     * @see WifiP2pManager.requestConnectionInfo
     */
    fun requestConnectionInfo(callback: (WifiP2pInfo) -> Unit = {}) =
        this.manager.requestConnectionInfo(this.channel, callback)

    /**
     * Obtain information about the current group.
     * @see WifiP2pManager.requestGroupInfo
     * @throws SecurityException if the permission has not been given
     */
    @Throws(SecurityException::class)
    fun requestGroupInfo(callback: (WifiP2pGroup?) -> Unit = {}) =
        this.manager.requestGroupInfo(this.channel, callback)

    /**
     * Create a new WiFi-Direct group.
     * @see WifiP2pManager.createGroup
     * @throws SecurityException if the permission has not been given
     */
    @Throws(SecurityException::class)
    fun createGroup(callback: () -> Unit = {}) =
        this.manager.createGroup(this.channel, object : WifiP2pManager.ActionListener {
            override fun onSuccess() { callback() }
            override fun onFailure(reason: Int) = throw BwdCreateGroupException(reason)
        })

    /**
     * Disconnect from the current WiFi-Direct group.
     * @see WifiP2pManager.removeGroup
     */
    fun removeGroup(callback: () -> Unit = {}) =
        this.manager.removeGroup(this.channel, object : WifiP2pManager.ActionListener {
            override fun onSuccess() { callback() }
            override fun onFailure(reason: Int) = throw BwdRemoveGroupException(reason)
        })

    /**
     * Create a new WiFi-Direct group. If already connected to a group, quit it first.
     *
     * Note: most of the failure on removal are caused by not having a group already created, which is checked.
     *
     * @param callback: the createGroup listener
     *
     * @see WifiP2pManager.createGroup
     * @see WifiP2pManager.removeGroup
     */
    fun recreateGroup(callback: () -> Unit = {}) {
        // get the current group information
        this.requestGroupInfo { group ->
            // if a group exist, quit it
            if (group != null)
                this.removeGroup { this@BwdManager.createGroup(callback) }
            else
                // create the group
                this.createGroup(callback)
        }
    }

    // Events

    val stateConnectionInfo = mutableStateOf<WifiP2pInfo?>(null)
    val statePeers = mutableStateOf<WifiP2pDeviceList?>(null)

    override fun onReceive(context: Context?, intent: Intent?) {
        // ignore empty intent
        if (intent == null)
            return

        // update the action corresponding state
        when (intent.action) {
            WifiP2pManager.WIFI_P2P_CONNECTION_CHANGED_ACTION -> this.requestConnectionInfo {
                connectionInfo -> stateConnectionInfo.value = connectionInfo
            }
            WifiP2pManager.WIFI_P2P_PEERS_CHANGED_ACTION -> this.requestPeers {
                peers -> statePeers.value = peers
            }
        }
        // TODO(Faraphel): implement event dispatcher
    }
}