package com.faraphel.tasks_valider.connectivity.task

import com.faraphel.tasks_valider.connectivity.task.api.TaskSessionManagerApi
import com.faraphel.tasks_valider.connectivity.task.session.TaskSessionManager
import com.faraphel.tasks_valider.database.TaskDatabase
import com.faraphel.tasks_valider.database.api.server.DatabaseApi
import com.faraphel.tasks_valider.database.entities.PersonEntity
import com.faraphel.tasks_valider.database.entities.SessionEntity
import fi.iki.elonen.NanoHTTPD


/**
 * A server to handle the task API to allow clients to interact with the database.
 * @param port the port of the server.
 * @param database the database to interact with.
 * @param session the current session.
 * @param adminPersonEntity the person that represent the host of the session.
 */
class TaskServer(
    private val port: Int,
    private val database: TaskDatabase,
    private val session: SessionEntity,
    private val adminPersonEntity: PersonEntity,
) : NanoHTTPD(port) {
    private val sessionManager = TaskSessionManager(adminPersonEntity)  ///< the session manager
    private val databaseApi = DatabaseApi(this.database, session)  ///< the api of the database
    private val sessionManagerApi = TaskSessionManagerApi(this.sessionManager, this.database)  ///< the api of the session manager

    /**
     * Get the admin person entity
     * @return the admin person entity
     */
    fun getAdminPersonEntity(): PersonEntity {
        return this.sessionManager.getAdminPersonEntity()
    }

    /**
     * Return a new client that can be used by the admin
     * @return the client
     */
    fun getAdminClient(): TaskClient {
        // create the session cookie for the admin
        val cookieSession = okhttp3.Cookie.Builder()
            .domain("localhost")
            .name("sessionToken")
            .value(this.sessionManager.adminSessionToken)
            .build()

        // create a new client
        return TaskClient(
            "localhost",
            this.port,
            listOf(cookieSession)
        )
    }

    /**
     * Handle an API request
     * @param httpSession the http session
     */
    override fun serve(httpSession: IHTTPSession): Response {
        // get the session data of the client
        val taskSessionData = this.sessionManager.getSessionData(httpSession)
        val taskSession = taskSessionData?.second

        // parse the url
        val uri: String = httpSession.uri.trim('/')
        val path = uri.split("/").toMutableList()

        // get the type of the request from the uri
        val requestType = path.removeFirstOrNull()
            ?: return newFixedLengthResponse(
                NanoHTTPD.Response.Status.BAD_REQUEST,
                "text/plain",
                "Missing request type"
            )

        // get the response from the correct part of the application
        return when (requestType) {
            // session requests
            "sessions" -> this.sessionManagerApi.handleRequest(taskSession, httpSession, path)
            // entities requests
            "entities" -> this.databaseApi.handleRequest(taskSession, httpSession, path)
            // invalid requests
            else ->
                newFixedLengthResponse(
                    NanoHTTPD.Response.Status.BAD_REQUEST,
                    "text/plain",
                    "Unknown request type"
                )
        }
    }

    /**
     * Start the server with the default configuration
     */
    override fun start() = super.start(SOCKET_READ_TIMEOUT, false)
}
