package com.faraphel.tasks_valider.connectivity.task.api

import com.faraphel.tasks_valider.connectivity.task.session.TaskPermission
import com.faraphel.tasks_valider.connectivity.task.session.TaskSession
import com.faraphel.tasks_valider.connectivity.task.session.TaskSessionManager
import com.faraphel.tasks_valider.database.TaskDatabase
import com.faraphel.tasks_valider.utils.parser
import com.google.gson.reflect.TypeToken
import fi.iki.elonen.NanoHTTPD


/**
 * the HTTP API for the session manager
 */
class TaskSessionManagerApi(
    private val sessionManager: TaskSessionManager,
    private val database: TaskDatabase
) {
    /**
     * Handle a HTTP Api request
     * @param taskSession the data of the client session
     * @param httpSession the data of the http session
     * @param path the path of the request
     */
    fun handleRequest(
        taskSession: TaskSession?,
        httpSession: NanoHTTPD.IHTTPSession,
        path: MutableList<String>,
    ): NanoHTTPD.Response {
        val action = path.removeFirstOrNull()

        return when (action) {
            "self" -> this.handleRequestSelf(taskSession, httpSession, path)
            "all" -> this.handleRequestAll(taskSession, httpSession, path)
            else ->
                NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.BAD_REQUEST,
                    "text/plain",
                    "Unknown action"
                )
        }
    }

    /**
     * Handle an HTTP Api request about the user own session
     */
    fun handleRequestSelf(
        taskSession: TaskSession?,
        httpSession: NanoHTTPD.IHTTPSession,
        path: MutableList<String>,
    ): NanoHTTPD.Response {
        when (httpSession.method) {
            // get the session data
            NanoHTTPD.Method.GET -> {
                // check if the session is valid
                if (taskSession == null)
                    return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.UNAUTHORIZED,
                        "text/plain",
                        "No session"
                    )

                // return the session data
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.OK,
                    "application/json",
                    parser.toJson(taskSession)
                )
            }

            // connect the user to the session
            NanoHTTPD.Method.POST -> {
                // get the user identifiers
                val identifiers: Map<String, String> = parser.fromJson(
                    httpSession.inputStream.bufferedReader().readText(),
                    object : TypeToken<Map<String, String>>() {}.type
                )

                // check for the id
                if (!identifiers.contains("id"))
                    return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.BAD_REQUEST,
                        "text/plain",
                        "Missing id"
                    )

                // get the id of the user (if invalid, return an error)
                val personId: Long = identifiers["id"]!!.toLongOrNull()
                    ?: return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.BAD_REQUEST,
                        "text/plain",
                        "Invalid id"
                    )

                // check for the password
                if (!identifiers.contains("password"))
                    return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.BAD_REQUEST,
                        "text/plain",
                        "Missing password"
                    )

                // check if the identifiers are correct
                val person = this.database.personDao().getById(personId)
                    ?: return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.BAD_REQUEST,
                        "text/plain",
                        "No person with this id"
                    )

                // check if the password is correct
                if (!person.checkPassword(identifiers["password"]!!))
                    return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.UNAUTHORIZED,
                        "text/plain",
                        "Invalid password"
                    )

                // create a new session for the userJHH
                val (sessionToken, session) = this.sessionManager.newSessionData(person)

                // create the response
                val response = NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.OK,
                    "text/plain",
                    "Session updated"
                )

                // set the session token in the cookies
                this.sessionManager.responseSetSessionData(
                    response,
                    httpSession.cookies,
                    sessionToken
                )

                // return the response
                return response
            }
            // ignore other methods
            else -> {
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
                    "text/plain",
                    "Invalid method"
                )
            }
        }
    }

    /**
     * Handle an HTTP Api request about all the sessions
     */
    private fun handleRequestAll(
        taskSession: TaskSession?,
        httpSession: NanoHTTPD.IHTTPSession,
        path: MutableList<String>,
    ): NanoHTTPD.Response {
        // check if the session is valid
        if (taskSession == null)
            return NanoHTTPD.newFixedLengthResponse(
                NanoHTTPD.Response.Status.UNAUTHORIZED,
                "text/plain",
                "No session"
            )

        // check the permission of the session
        if (taskSession.person.role.permissions.contains(TaskPermission.ADMIN))
            return NanoHTTPD.newFixedLengthResponse(
                NanoHTTPD.Response.Status.FORBIDDEN,
                "text/plain",
                "You are not allowed to update a session"
            )

        when (httpSession.method) {
            // change a specific client session data
            NanoHTTPD.Method.POST -> {
                // parse the content of the request
                val targetSession = parser.fromJson(
                    httpSession.inputStream.bufferedReader().readText(),
                    TaskSession::class.java
                )

                val targetSessionToken = path.removeFirstOrNull()
                    ?: return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.BAD_REQUEST,
                        "text/plain",
                        "Missing session token"
                    )

                // update the session
                this.sessionManager.updateSessionData(targetSessionToken, targetSession)

                // success message
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.OK,
                    "text/plain",
                    "Session updated"
                )
            }

            // delete the session
            NanoHTTPD.Method.DELETE -> {
                val targetSessionToken = path.removeFirstOrNull()
                    ?: return NanoHTTPD.newFixedLengthResponse(
                        NanoHTTPD.Response.Status.BAD_REQUEST,
                        "text/plain",
                        "Missing session token"
                    )

                // delete the target session
                this.sessionManager.deleteSessionData(targetSessionToken)

                // success message
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.OK,
                    "text/plain",
                    "Session deleted"
                )
            }

            // ignore other methods
            else -> {
                return NanoHTTPD.newFixedLengthResponse(
                    NanoHTTPD.Response.Status.METHOD_NOT_ALLOWED,
                    "text/plain",
                    "Invalid method"
                )
            }
        }
    }
}
