package com.faraphel.tasks_valider.ui.screen.communication

import android.app.Activity
import android.os.Build
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.compose.foundation.layout.*
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.navigation.NavController
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import com.faraphel.tasks_valider.connectivity.bwd.BwdManager
import com.faraphel.tasks_valider.database.TaskDatabase
import com.faraphel.tasks_valider.ui.screen.communication.internet.CommunicationInternetSelectScreen
import com.faraphel.tasks_valider.ui.screen.communication.wifiP2p.CommunicationWifiP2pScreen


/**
 * CommunicationController is the main controller for the communication screen.
 * It is responsible for handling the navigation between the different communication methods.
 * It is also responsible for initializing the communication methods.
 *
 * @param activity: The activity that hosts the communication screen.
 * @param database: the database.
 */
@RequiresApi(Build.VERSION_CODES.O)
@Composable
fun CommunicationModeSelectionScreen(activity: Activity, database: TaskDatabase) {
    val controller = rememberNavController()

    NavHost(
        navController = controller,
        startDestination = "select"
    ) {
        composable("select") {
            CommunicationSelectContent(controller, activity)
        }
        composable("internet") {
            CommunicationInternetSelectScreen(activity, database)
        }
        composable("wifi-p2p") {
            val bwdManager = BwdManager.fromActivity(activity)
            CommunicationWifiP2pScreen(activity, bwdManager)
        }
    }
}


/**
 * Communication screen that allows the user to choose the communication mode
 */
@Composable
fun CommunicationSelectContent(controller: NavController, activity: Activity) {
    val isWifiP2pSupported = BwdManager.isSupported(activity)

    Column(
        modifier = Modifier.fillMaxSize(),
        verticalArrangement = Arrangement.Center,
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        // title
        Text(
            text = "Connection Type",
            fontSize = 32.sp
        )

        // separator
        Spacer(modifier = Modifier.height(24.dp))
        
        // internet communication mode
        Button(onClick = { controller.navigate("internet") }) {
            Text("Internet")
        }

        // wifi-direct communication mode
        Button(
            colors = ButtonDefaults.buttonColors(
                // if the WiFi-Direct is not supported, the button is grayed out
                containerColor = if (isWifiP2pSupported) MaterialTheme.colorScheme.primary else Color.Gray
            ),
            onClick = {
                // if the WiFi-Direct is supported, navigate to the WiFi-Direct screen
                if (isWifiP2pSupported) controller.navigate("wifi-p2p")
                // if the WiFi-Direct is not supported, show a toast message
                else Toast.makeText(activity, "WiFi-Direct is not supported on this device", Toast.LENGTH_SHORT).show()
            }
        ) {
            Text("WiFi-Direct")
        }
    }
}
