package com.faraphel.tasks_valider.ui.screen.task

import android.app.Activity
import android.widget.Toast
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.navigation.NavController
import com.faraphel.tasks_valider.connectivity.task.TaskClient
import com.faraphel.tasks_valider.database.entities.PersonEntity
import com.faraphel.tasks_valider.database.entities.ValidationEntity
import com.faraphel.tasks_valider.ui.screen.scan.qr.ScanBarcodeScreen
import com.journeyapps.barcodescanner.BarcodeResult
import okhttp3.HttpUrl.Companion.toHttpUrl


@Composable
fun QuickValidationScreen(
    controller: NavController,
    activity: Activity,
    client: TaskClient,
    user: PersonEntity,
) {
    val barcode = remember { mutableStateOf<BarcodeResult?>(null) }

    // prompt for the qr code if not found
    if (barcode.value == null)
        return ScanBarcodeScreen(activity, barcode)

    // show the content of the qr code
    val studentUrl = barcode.value!!.text.toHttpUrl()
    val cardId = studentUrl.pathSegments[0]

    // when the barcode changed
    LaunchedEffect(cardId) {
        Thread {
            quickValidation(
                controller,
                activity,
                client,
                user,
                cardId,
            )
        }.start()
    }
}


/**
 * Validate the latest task of a user from its student card
 */
fun quickValidation(
    controller: NavController,
    activity: Activity,
    client: TaskClient,
    user: PersonEntity,
    cardId: String,
) {
    // action when an error occurred or everything worked
    fun finish() {
        activity.runOnUiThread {
            // go back to the main screen
            controller.navigateUp()
        }
    }

    // requests all the persons
    val allPersons = client.personApi.getAll()
    // get the person with the matching card
    val person = allPersons.firstOrNull { person -> person.cardId == cardId }

    if (person == null) {
        // tell to the user that this card is linked to nobody
        activity.runOnUiThread {
            Toast.makeText(activity, "No person found for that card.", Toast.LENGTH_LONG).show()
        }
        return finish()
    }

    // requests all the relation persons - subjects
    val allRelationsPersonSubject = client.relationPersonSessionSubjectApi.getAll()
    // get the corresponding relation
    val relationPersonSubject = allRelationsPersonSubject.first { relation -> relation.studentId == person.id }

    // requests all the tasks
    val allTasks = client.taskApi.getAll()
    // get the corresponding tasks
    val tasks = allTasks
        .filter { task -> task.subjectId == relationPersonSubject.subjectId }
        .sortedBy { task -> task.order }

    // requests all the validations
    val allValidations = client.validationApi.getAll()
    // get the corresponding relation
    val validations = allValidations.filter { validation -> validation.studentId == person.id }

    // get the first task without any validation
    val task = tasks.firstOrNull { task ->
        // check in all the validations if the task is found
        val validation = validations.firstOrNull { validation -> validation.taskId == task.id }
        // keep the task if it has no validation
        validation == null
    }

    if (task == null) {
        // tell to the user the action cannot be done
        activity.runOnUiThread {
            Toast.makeText(activity, "There are no tasks left.", Toast.LENGTH_LONG).show()
        }
        return finish()
    }

    // create a new validation on the server
    client.validationApi.save(
        ValidationEntity(
            teacherId=user.id,
            studentId=person.id,
            taskId=task.id,
        )
    )
    // confirm to the user the action was successful
    activity.runOnUiThread {
        Toast.makeText(activity, "Validated \"${task.title}\".", Toast.LENGTH_LONG).show()
    }

    return finish()
}